-- VECTRIC LUA SCRIPT
require("mobdebug").start()

g_version = "1.0"
g_title   = "Flip Board Maker"

function DrawLine(line, cur_point, fingers, finger_vec, vec_tool, finger_clearance_vec, swap_order, toggle)

	for n = 1, fingers*2 do
    
    if swap_order then
      cur_point = (cur_point + toggle*vec_tool) + toggle*finger_clearance_vec
      line:LineTo(cur_point)
      
      cur_point = cur_point + finger_vec
      line:LineTo(cur_point)
    else
      cur_point = cur_point + finger_vec
      line:LineTo(cur_point)
      
      cur_point = (cur_point + toggle*vec_tool) + toggle*finger_clearance_vec
      line:LineTo(cur_point)
      
    end
    
    toggle = -toggle
  
	end
	
	return cur_point
end

function DrawBoard(Width, Height, fingersX, fingersY, tool_width, finger_clearance)
	-- initialize drawing calculations
	local vec_x = Vector2D(Width / (fingersX*2), 0)
	local vec_y = Vector2D(0, Height / (fingersY*2))
	
  -- Create the flip board geometry
	local cur_point = Point2D(0, 0)
	local line = Contour(0.0)
	line:AppendPoint(cur_point)
	cur_point = DrawLine(line, cur_point, fingersX, vec_x, Vector2D(0, -1*tool_width), Vector2D(0, -1*finger_clearance), true, 1)
	cur_point = DrawLine(line, cur_point, fingersY, vec_y, Vector2D(-1*tool_width, 0), Vector2D(-1*finger_clearance,0 ), false, -1)
	cur_point = DrawLine(line, cur_point, fingersX, -1*vec_x, Vector2D(0, tool_width), Vector2D(0, finger_clearance), true, 1)
	cur_point = DrawLine(line, cur_point, fingersY, -1*vec_y, Vector2D(tool_width, 0), Vector2D(finger_clearance, 0), false, -1)
	local line_object = CreateCadContour(line)

  -- Get 2D Center of Material
  local mtl_bb = MaterialBlock().MaterialBox    -- Box2D(mtl_block.MaterialBox.X, mtl_block.MaterialBox.Y)
  local mtl_bounding_box_2D_center = Box2D(Point2D(mtl_bb.MinX, mtl_bb.MinY), Point2D(mtl_bb.MaxX, mtl_bb.MaxY)).Center
  
  -- Get 2D Center of the flip board object
  local line_object_bounding_box_center = line_object:GetBoundingBox().Center
  
  -- Calculate translation vector to center flip board
  local translation_matrix = TranslationMatrix2D(mtl_bounding_box_2D_center - line_object_bounding_box_center)
  
  -- Translate the CadContour (i.e. center it)
  line_object:Transform(translation_matrix)
  
--[[  
  -- Prevent scaling
  line_object:CanTransform(1+4) -- Only returns true/false
--]]

  -- Select and display the flip board
	layer:AddObject(line_object, true)	
  
  job.Selection:Clear()
  job.Selection:Add(line_object, true, false)
	job:Refresh2DView()
  return true
end

function OnLuaButton_MinMaxDimensionsOuterID(dialog)
  dialog:UpdateLabelField("MinMaxWidthID", "Max Outside");
  dialog:UpdateLabelField("MinMaxHeightID", "Max Outside");
  return true
end

function OnLuaButton_MinMaxDimensionsInnerID(dialog)  
  dialog:UpdateLabelField("MinMaxWidthID", "Min Inside");
  dialog:UpdateLabelField("MinMaxHeightID", "Min Inside");
  return true
end

--[[
function OnLuaButton_DispCutBoundariesID(dialog)  
  return true
end
--]]

function SetupDialog(dialog)
  local mtl_block = MaterialBlock()
  
  local inset = 2       -- reduce size of flip board by 2 inches vs size of material block by default
  local ToolDia = 0.25  -- assume we are using a 1/4" diameter endmill
  local FingerClearance = 0.000
  local units = "inches"
  if mtl_block.InMM then
    inset = 50      -- if using mm, then reduce size by 50mm
    ToolDia = 6.0   -- if using mm, then assume a 6mm diameter endmill
    FingerClearance = 0.00
    units = "mm"    -- set units
  end
  
  dialog:AddLabelField("GadgetTitle", g_title)
	dialog:AddLabelField("GadgetVersion", g_version)

  dialog:AddLabelField("MinMaxWidthID", "Max Outside")   -- improvement: get from stock!
  dialog:AddDoubleField("WidthID", mtl_block.Width - inset)
  dialog:AddLabelField("WidthUnitsID", units)
  
  dialog:AddLabelField("MinMaxHeightID", "Max Outside")    -- improvement: get from stock!
  dialog:AddDoubleField("HeightID", mtl_block.Height - inset)
  dialog:AddLabelField("HeightUnitsID", units)

  dialog:AddRadioGroup("Qualifier", 1)

  dialog:AddIntegerField("NumFingersXID", 2)
  dialog:AddIntegerField("NumFingersYID", 2)
  
  dialog:AddDoubleField("FingerClearanceID", FingerClearance)
  dialog:AddLabelField("FingerClearanceUnitsID", units)
  
  dialog:AddDoubleField("ToolDiaID", ToolDia)
  dialog:AddLabelField("ToolDiaUnitsID", units)
  
  -- dialog:AddCheckBox("DispCutBoundariesID", false)

  return true
end

function ProcessDialog(dialog)
  
  local Width             = dialog:GetDoubleField("WidthID")
  local Height            = dialog:GetDoubleField("HeightID")
  local QualifierIndex    = dialog:GetRadioIndex("Qualifier")
  local NumFingersX       = dialog:GetIntegerField("NumFingersXID")
  local NumFingersY       = dialog:GetIntegerField("NumFingersYID")
  local FingerClearance   = dialog:GetDoubleField("FingerClearanceID")
  local ToolDia           = dialog:GetDoubleField("ToolDiaID")
  -- local DispCutBoundaries = dialog:GetCheckBox("DispCutBoundariesID")

  if QualifierIndex == 1 then
    Width = Width - 3*ToolDia + 2*FingerClearance
    Height = Height - 3*ToolDia + 2*FingerClearance
  end
  
  if QualifierIndex == 2 then
    Width = Width + 1*ToolDia + 0*FingerClearance
    Height = Height + 1*ToolDia + 0*FingerClearance
  end

  DrawBoard(Width, Height, NumFingersX, NumFingersY, ToolDia, -FingerClearance)

  return true
end

function main(path)
	-- Get the current job
	job = VectricJob()
	if not job.Exists then
		DisplayMessageBox("No job loaded")
		return false
	end
	
	-- Get the active layer
	layer = job.LayerManager:GetActiveLayer()
	if layer == nil then
		DisplayMessageBox("No active layer")
		return false
	end

  -- Setup and show the Dialog Box
  local html_path = "file:" .. path .. "\\Flip_Board_Maker.htm"
  local dialog = HTML_Dialog(false, html_path, 500, 750, "Flip Board Maker")
  SetupDialog(dialog)
  if not dialog:ShowDialog() then
    return false
  end
  ProcessDialog(dialog)

	return true
end

